﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Web;
namespace Sense
{

    /// <summary>
    /// 1. 接口说明参考demo\WebServer\WebServer2018-12-10\VirboxWeb服务接口说明.docx
    /// 2. “描述” 指的是用json字符串组织的各种锁类型的会话信息
    /// 3. 需要带有web_server接口的Virbox用户工具
    /// 4. 接口需要引用 Newtonsoft.Json, 和 System.web 库
    /// 
    /// </summary>
    class SenseWebServer
    {

        /// <summary>
        /// 深思Web Server服务后台端口号,默认为9080, 可通过在Web Server安装目录下的websrv_config.ini文件中修改。修改后需要重启VirboxWebServer服务
        /// </summary>
        public static int port = 9080;

        /// <summary>
        /// 使用http 或 https协议，值为true时,需要设置 https_valid_callback属性
        /// 默认为true,使用https协议。 可通过在Web Server安装目录下的websrv_config.ini文件中修改。修改后需要重启服务
        /// </summary>
        public static bool use_https = true;

        /// <summary>
        /// 设置用于https校验的回调， 只有在use_https 为true时， 该值才有效
        /// </summary>
        public static RemoteCertificateValidationCallback https_valid_callback = null;


        private static string host = "localhost";
        private static string path = "/v1/api";

        private static string make_schema() {
            return use_https ? "https://" : "http://";
        }


        /// <summary>
        /// 基础http请求
        /// </summary>
        /// <param name="url"></param>  
        /// <param name="method"> GET /POST </param>  
        /// <param name="requestdata"> 只在method为"POST"时有意义。 post数据的byte数组 </param> 
        /// <returns></returns>
        private static string SenseRequest(string url, string method, byte[] requestdata)
        {
            if (use_https)
            {
                ServicePointManager.ServerCertificateValidationCallback = https_valid_callback;
            }

            WebRequest request = WebRequest.Create(url);
            if (method == "POST")
            {
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoed";
            }
            else
                request.Method = "GET";

            Stream smrequest = null;
            Stream smresponse = null;
            StreamReader reader = null;
            try
            {
                if (method == "POST")
                {
                    smrequest = request.GetRequestStream();
                    smrequest.Write(requestdata, 0, requestdata.Length);
                }

                smresponse = request.GetResponse().GetResponseStream();
                reader = new StreamReader(smresponse);
                return reader.ReadToEnd();
            }
            catch (Exception ex)
            {
                throw;
            }
            finally
            {
                if (reader != null)
                    reader.Close();
                if (smrequest != null)
                    smrequest.Close();
                if (smresponse != null)
                    smresponse.Close();
            }

        }

        /// <summary>
        /// 检查SenseRequest的返回值
        /// </summary>
        /// <param name="rsp"></param>
        /// <param name="out_rsp"></param>
        /// <returns>
        /// 解析成功后会检查最外层json['code', 'desc', 'data'], 
        ///      如果code = 0, out_rsp 为错误描述
        ///      如果code != 0, out_rsp 为data数据段的字符串
        /// </returns>
        private static int check_rsp_json(string rsp, ref string out_rsp)
        {
            JObject rsp_json = (JObject)JsonConvert.DeserializeObject(rsp);
            if ((int)rsp_json["code"] != 0)
                out_rsp = rsp_json["desc"].ToString();
            else
                out_rsp = (string)rsp_json["data"];

            return (int)rsp_json["code"];
        }

        /// <summary>
        /// 返回当前后台服务的版本号，以及web_server的版本号
        /// </summary>
        /// <param name="out_rsp"></param>
        /// <returns></returns>
        public static int slm_get_version(ref string out_rsp)
        {
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/wbsrv/getVersion", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 查询本地硬件锁描述
        /// </summary>
        /// <param name="out_rsp">找到的指定描述的json数组字符串</param>  
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_get_local_description(ref string out_rsp)
        {
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_get_local_description", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 查询本地硬件锁以及网络锁描述
        /// </summary>
        /// <param name="out_rsp">找到的指定描述的json数组字符串</param>  
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_get_all_description(ref string out_rsp)
        {
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_get_all_description", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 查询本地硬件锁信息
        /// </summary>
        /// <param name="local_desc">指定的硬件锁描述</param>  
        /// <param name="out_rsp">找到的指定描述的json字符串</param>  
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_get_device_info(string local_desc, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(local_desc);
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_get_device_info", "POST", 
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取设备证书，支持硬件锁/云锁 
        /// </summary>
        /// <param name="local_desc">需要获取证书的锁描述字符串</param>
        /// <param name="out_rsp">找到的指定描述的json字符串</param>  
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_ctrl_get_device_cert(string local_desc, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(local_desc);
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_ctrl_get_device_cert", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 登录账号
        /// </summary>
        /// <param name="username"> 账号名, 最大16位 </param>   
        /// <param name="password"> 密码， 最大16位 </param>  
        /// <param name="out_rsp"> 成功返回具体值, 失败返回错误描述 </param>    
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_cloud_user_login(string username, string password, ref string out_rsp)
        {
            string requestdata = "username=" + HttpUtility.UrlEncode(username) + "&";
            requestdata += "password=" + HttpUtility.UrlEncode(password) + "";

            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/cloudUser/ss_cloud_user_login", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 退出账号
        /// </summary> 
        /// <param name="user_guid">slm_cloud_user_login 返回的id字符串</param>  
        /// <param name="out_rsp">成功返回具体值, 失败返回错误描述 </param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_cloud_user_logout(string user_guid, ref string out_rsp)
        {
            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/cloudUser/ss_cloud_user_logout", "POST",
                Encoding.Default.GetBytes("user_guid=" + user_guid));

            return check_rsp_json(rsp, ref out_rsp);
        }


        /// <summary>
        /// 查询所有云/软锁描述
        /// </summary>
        /// <param name="user_guid">slm_cloud_user_login返回的user_guid</param>   
        /// <param name="out_rsp">找到的指定描述的json数组字符串</param>  
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_get_cloud_description(string user_guid, ref string out_rsp)
        {
            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/control/slm_get_cloud_description", "POST",
                Encoding.Default.GetBytes("user_guid=" + user_guid));

            return check_rsp_json(rsp, ref out_rsp);
        }


        /// <summary>
        /// 查询指定开发商ID的未绑定过软锁描述(自封接口）
        /// </summary>
        /// <param name="developer_id">开发商id字符串</param>
        /// <param name="user_guid">slm_cloud_user_login返回的user_guid</param>
        /// <param name="out_rsp">找到的指定描述的json数组字符串, 未找到时out_rsp不变</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_cloud_find_slock_desc(string developer_id, string user_guid, ref string out_rsp)
        {
            string desc_array_str = null;
            int ret = 0;
            if ((ret = slm_get_cloud_description(user_guid, ref desc_array_str)) != 0)
            {
                out_rsp = desc_array_str;
                return ret;
            }
            JArray desc_array = (JArray)JsonConvert.DeserializeObject(desc_array_str);
            foreach (JObject desc in desc_array)
            {
                if (desc["developer_id"].ToString() == developer_id
                    && desc["user_guid"].ToString() == user_guid
                    && desc["type"].ToString() == "smart_offline")
                {
                    if (out_rsp != null)
                        out_rsp = JsonConvert.SerializeObject(desc);
                    break;
                }
            }

            return 0;
        }


        /// <summary>
        /// 查询某个desc描述中的所有许可信息
        /// </summary>
        /// <param name="desc"> 通过 slm_get_local_description  slm_get_cloud_description获取到的某个指定desc</param>
        /// <param name="out_rsp">返回的许可json数组字符串, 未找到时out_rsp不变</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_read_brief_license_context(string desc, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(desc);
            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/control/slm_read_brief_license_context", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }


        /// <summary>
        /// 软锁在线绑定
        /// </summary>
        /// <param name="smart_cloud_desc">使用slm_get_cloud_description 获取到的描述, 与指定的developer_id 和 user_guid对应</param>
        /// <param name="license_id">将要绑定的许可id</param>
        /// <param name="out_rsp">绑定成功该值为空， 错误时为错误描述</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_bind_offline_license(string smart_cloud_desc, UInt32 license_id, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(smart_cloud_desc) + "&" + "license_id=" + license_id;

            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/control/slm_bind_offline_license", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取所有已绑定软锁描述
        /// </summary>
        /// <param name="out_rsp">查询出的该机器上当前已绑定软锁的json数组字符串</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_get_offline_desc(ref string out_rsp)
        {
            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/control/slm_get_offline_desc", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 软锁在线解绑
        /// </summary>
        /// <param name="smart_offline_desc">将要解绑的软锁描述， 需要与developer_id, user_guid 对应</param>
        /// <param name="license_id">将要解绑的软锁许可</param>
        /// <param name="out_rsp">绑定成功该值为空， 错误时为错误描述</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_unbind_offline_license(string smart_offline_desc, UInt32 license_id, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(smart_offline_desc) + "&" + "license_id=" + license_id;

            string rsp = SenseRequest(  make_schema() + host + ":" + port + path + "/control/slm_unbind_offline_license", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取软锁离线本机c2d信息
        /// </summary>
        /// <param name="out_rsp">json字符串格式的数据，</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_offline_bind_c2d(ref string out_rsp)
        {
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_offline_bind_c2d", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取软锁离线绑定d2c数据
        /// </summary>
        /// <param name="smart_offline_desc">使用slm_get_cloud_description获取到的描述， 需要与developer_id, user_guid 对应</param>
        /// <param name="c2d_content">slm_offline_bind_c2d 获取到的本机硬件指纹</param>
        /// <param name="license_id">将要解绑的软锁许可</param>
        /// <param name="out_rsp">绑定成功该值为空， 错误时为错误描述</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_cloud_offline_get_d2c(string smart_offline_desc, string c2d_content, UInt32 license_id, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(smart_offline_desc) 
                + "&" + "license_id=" + license_id 
                + "&" + "c2d=" + HttpUtility.UrlEncode(c2d_content);
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_cloud_offline_get_d2c", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }


        /// <summary>
        /// 软锁离线绑定d2c
        /// </summary>
        /// <param name="slock_d2c"> 通过 slm_cloud_offline_get_d2c  获取到的数据包</param>
        /// <param name="out_rsp">绑定成功该值为空， 错误时为错误描述</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_offline_bind_d2c(string slock_d2c, ref string out_rsp)
        {
            string requestdata = "d2c=" + HttpUtility.UrlEncode(slock_d2c);
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_offline_bind_d2c", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }


        /// <summary>
        /// 升级硬件锁数据包
        /// </summary>
        /// <param name="hlock_d2c"> 通过Openapi 或文件读取到的d2c数据</param>
        /// <param name="out_rsp">绑定成功该值为空， 错误时为错误描述</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int slm_update_hlock_d2c(string hlock_d2c, ref string out_rsp)
        {
            string requestdata = "d2c=" + HttpUtility.UrlEncode(hlock_d2c);
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/update/slm_update", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取当前计算机插入硬件锁设备证书链
        /// </summary>
        /// <param name="out_rsp">查询出的该机器上当前硬件锁设备证书链的json数组字符串</param>
        /// <returns>
        ///     成功返回0， 失败返回错误码
        /// </returns>
        public static int getLocalDeviceP7B(ref string out_rsp)
        {
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/ext/getLocalDeviceP7B", "GET", null);

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lock_desc">加密锁描述</param>
        /// <param name="verify_data">verify_data需要使用base64编码并且在添加SENSELOCK前缀， 真实的签名数据为: SENSELOCK{VERIFY_DATA}</param>
        /// <param name="out_rsp"></param>
        /// <returns></returns>
        public static int slm_ctrl_sign_by_device(string lock_desc, string verify_data, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(lock_desc) + "&verify_data=" + verify_data;

            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_get_pub_size", "POST",
                Encoding.Default.GetBytes(requestdata));
            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取加密锁内公开区数据大小
        /// </summary>
        /// <param name="lock_desc"> 加密锁描述</param>
        /// <param name="license_id">许可id</param>
        /// <param name="out_rsp"></param>
        /// <returns></returns>
        public static int slm_get_pub_size(string lock_desc, UInt32 license_id, ref string out_rsp) {
            string requestdata = "desc=" + HttpUtility.UrlEncode(lock_desc) + "&" + "license_id=" + license_id;

            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_get_pub_size", "POST",
                Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }

        /// <summary>
        /// 获取加密锁公开区数据
        /// </summary>
        /// <param name="lock_desc"> 加密锁描述</param>
        /// <param name="license_id">许可id</param>
        /// <param name="offset">起始偏移</param>
        /// <param name="length">长度</param>
        /// <param name="out_rsp"></param>
        /// <returns></returns>
        public static int slm_read_pub_data(string lock_desc, UInt32 license_id, int offset, int length, ref string out_rsp)
        {
            string requestdata = "desc=" + HttpUtility.UrlEncode(lock_desc) + "&" + "license_id=" + license_id 
                                + "&offset=" + offset + "&length=" + length;
            string rsp = SenseRequest(make_schema() + host + ":" + port + path + "/control/slm_read_pub_data", "POST",
               Encoding.Default.GetBytes(requestdata));

            return check_rsp_json(rsp, ref out_rsp);
        }
    }
}
